/*
 * adxl335.c
 *
 *  Created on: Dec 26, 2022
 *      Author: hussamaldean
 */


#include "stm32f4xx.h"
#include "adxl335.h"

#define LEN_3_CH 0x02

#define CH0 0x00
#define CH1 0x01
#define CH4 0x04

#define RawMax  4095

uint16_t acc_data[3];


void adxl335_init()
{
	/*Enable clock access to GPIOA*/
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN;
	/*Configure PA0, PA1 and PA4 as analog*/
	GPIOA->MODER|=GPIO_MODER_MODE0|GPIO_MODER_MODE1|GPIO_MODER_MODE4;

	/*Enable clock access to ADC1*/
	RCC->APB2ENR|=RCC_APB2ENR_ADC1EN;

	/*Enable scan mode */

	ADC1->CR1|=ADC_CR1_SCAN;


	/*Enable the following:
	 * DMA request.
	 * Continuous mode.
	 * */

	ADC1->CR2|=ADC_CR2_CONT|ADC_CR2_DDS|ADC_CR2_DMA;

	/*Set the length of conversion to be 3.*/
	ADC1->SQR1|=(LEN_3_CH<<ADC_SQR1_L_Pos);

	/*Set the sequence as following:
	 * PA0(CH0), PA1(CH1), PA4(CH4)
	 * */

	ADC1->SQR3|=(CH0<<ADC_SQR3_SQ1_Pos)|(CH1<<ADC_SQR3_SQ2_Pos)|(CH4<<ADC_SQR3_SQ3_Pos);

	/*DMA Setup*/

	/*DMA related setup*/
	RCC->AHB1ENR		|=RCC_AHB1ENR_DMA2EN;
	DMA2_Stream0->CR	&=~DMA_SxCR_EN;
	while(DMA2_Stream0->CR ==DMA_SxCR_EN){;}
	DMA2_Stream0->CR	|=DMA_SxCR_MSIZE_0|DMA_SxCR_PSIZE_0|DMA_SxCR_MINC|DMA_SxCR_CIRC;
	DMA2_Stream0->PAR	 =(uint32_t)(&(ADC1->DR));
	DMA2_Stream0->M0AR 	 =(uint32_t )(acc_data);
	DMA2_Stream0->NDTR	 =3;

	/*Launch the DMA*/
	DMA2_Stream0->CR 	|=DMA_SxCR_EN;

	/*Launch the ADC*/
	ADC1->CR2			|=ADC_CR2_ADON;

	ADC1->CR2|=ADC_CR2_SWSTART;


}



static float adc_to_acc(uint16_t adc_value)
{
	return  ((( ( (float)(adc_value * 3.3)/RawMax) - 1.65 ) / 0.330)*9.81);

}


void get_accel_data(acc_data_t *acc)
{

	float ax_scalled=adc_to_acc(acc_data[0]);
	float ay_scalled=adc_to_acc(acc_data[1]);
	float az_scalled=adc_to_acc(acc_data[2]);


	acc->accx=ax_scalled;
	acc->accy=ay_scalled;
	acc->accz=az_scalled;


}
